require 'httparty'
require 'awesome_print'
require 'tzinfo'

# required to sign requests
require 'openssl'
require 'base64'

# include mws modules
require 'mws/address'
require 'mws/request'
require 'mws/packing_slip'
require 'mws/fba_label'

# barcodes
require 'barby'
require 'barby/outputter/png_outputter'

##
# MWS
# 
module MWS
  
  ##
  # MWS::Client
  # 
  class Client
    def initialize(args={})
      args.each { |k, v| send("#{k}=", v) }
      @items = []
    end
    
    # Amazon Access Key ID
    attr_accessor :access_key_id
    
    # Amazon Secret Access Key
    attr_accessor :secret_access_key
    
    # Amazon Merchant ID (also Seller ID)
    attr_accessor :merchant_id
    
    # Amazon Marketplace
    attr_accessor :marketplace_id
    
    # From Address
    attr_accessor :from_address
    
    # Label Prep Preference
    attr_accessor :label_prep_preference
    
    # Items
    attr_accessor :items
    
    # Inbound Shipment
    attr_accessor :inbound_shipment, :inbound_shipment_plan
    
    # Shipment Name
    attr_accessor :shipment_name
    
    # Shipment ID (assigned by amazon)
    attr_accessor :shipment_id
    
    attr_accessor :s3_access_key_id, :s3_secret_access_key
    
    attr_accessor :shipment_status
    
    attr_accessor :fulfillment_center_id
    
    ##
    # GetServiceStatus
    # 
    def get_service_status
      Request.new({
        'Action' => 'GetServiceStatus',
        'Marketplace' => @marketplace_id,
        'SellerId' => @merchant_id,
        'AWSAccessKeyId' => @access_key_id
      },@secret_access_key).post
    end
    
    ##
    # CreateInboundShipmentPlan
    # 
    def create_inbound_shipment_plan
      request = Request.new({
        'Action'                              => 'CreateInboundShipmentPlan',
        'Marketplace'                         => @marketplace_id,
        'SellerId'                            => @merchant_id,
        'AWSAccessKeyId'                      => @access_key_id,
        'ShipFromAddress.Name'                => @from_address[:name],
        'ShipFromAddress.AddressLine1'        => @from_address[:address_line_1],
        'ShipFromAddress.AddressLine2'        => @from_address[:address_line_2],
        'ShipFromAddress.City'                => @from_address[:city],
        'ShipFromAddress.StateOrProvinceCode' => @from_address[:state_or_province_code],
        'ShipFromAddress.PostalCode'          => @from_address[:postal_code],
        'ShipFromAddress.CountryCode'         => @from_address[:country_code],
        'LabelPrepPreference'                 => @label_prep_preference
      },@secret_access_key)
      
      @items.flatten!.each_with_index do |item, index|
        request << {
          "InboundShipmentPlanRequestItems.member.#{index+1}.Condition" => item[:condition],
          "InboundShipmentPlanRequestItems.member.#{index+1}.SellerSKU" => item[:seller_sku],
          "InboundShipmentPlanRequestItems.member.#{index+1}.ASIN"      => item[:asin],
          "InboundShipmentPlanRequestItems.member.#{index+1}.Quantity"  => item[:quantity]
        }
      end
      
      @inbound_shipment_plan = request.post["CreateInboundShipmentPlanResponse"]["CreateInboundShipmentPlanResult"]["InboundShipmentPlans"]
      @fulfillment_center_id = @inbound_shipment_plan["member"]["DestinationFulfillmentCenterId"]
      @inbound_shipment_plan
    end
    
    ##
    # CreateInboundShipment
    # 
    def create_inbound_shipment
      puts "creating new inbound shipment..."
      
      request = Request.new({
        'Action'                                                    => 'CreateInboundShipment',
        'Marketplace'                                               => @marketplace_id,
        'SellerId'                                                  => @merchant_id,
        'AWSAccessKeyId'                                            => @access_key_id,
        'InboundShipmentHeader.ShipmentName'                        => @shipment_name,
        'InboundShipmentHeader.ShipFromAddress.Name'                => @from_address[:name],
        'InboundShipmentHeader.ShipFromAddress.AddressLine1'        => @from_address[:address_line_1],
        'InboundShipmentHeader.ShipFromAddress.AddressLine2'        => @from_address[:address_line_2],
        'InboundShipmentHeader.ShipFromAddress.City'                => @from_address[:city],
        'InboundShipmentHeader.ShipFromAddress.StateOrProvinceCode' => @from_address[:state_or_province_code],
        'InboundShipmentHeader.ShipFromAddress.PostalCode'          => @from_address[:postal_code],
        'InboundShipmentHeader.ShipFromAddress.CountryCode'         => @from_address[:country_code],
        'InboundShipmentHeader.DestinationFulfillmentCenterId'      => @inbound_shipment_plan["member"]["DestinationFulfillmentCenterId"]
      },@secret_access_key)
      
      puts "created request"

      @inbound_shipment_plan["member"]["Items"]["member"].each_with_index do |item, index|
        request << {
          "InboundShipmentItems.member.#{index+1}.QuantityShipped"  => item["Quantity"],
          "InboundShipmentItems.member.#{index+1}.SellerSKU"        => item["SellerSKU"]
        }
      end
      
      puts "added items"
      response = request.post.to_hash
      ap response
      @shipment_id = request.post["CreateInboundShipmentResponse"]["CreateInboundShipmentResult"]["ShipmentId"]
    end
    
    def format_from_address
      [
        @from_address[:name],
        [@from_address[:address_line_1], @from_address[:address_line_2]].join,
        "#{@from_address[:city]}, #{@from_address[:state_or_province_code]} #{@from_address[:postal_code]}",
        TZInfo::Country.get(@from_address[:country_code]).name
      ].join("\n")
    end
    
    def format_to_address
      ship_to_address = @inbound_shipment_plan["member"]["ShipToAddress"]

      [
        ship_to_address["Name"],
        [ship_to_address["AddressLine1"], (ship_to_address["AddressLine2"] || [])].join,
        "#{ship_to_address["City"]}, #{ship_to_address["StateOrProvinceCode"]} #{ship_to_address["PostalCode"]}",
        TZInfo::Country.get(ship_to_address["CountryCode"]).name
      ].join("\n")
    end
    
    def fba_label
      FBA::ShippingLabel.new({
        :shipment_id => @shipment_id,
        :ship_from_address => format_from_address,
        :ship_to_address => format_to_address,
        :access_key_id => @s3_access_key_id,
        :secret_access_key => @s3_secret_access_key,
        :template_url => "http://bsgnpo.s3.amazonaws.com/template.fba.label.pdf",
        :bucket => 'bsgnpo'
      })
    end
    
    def fba_packing_slip
      MWS::PackingSlip.new({
        :shipment_id => @shipment_id,
        :ship_from_address => format_from_address,
        :ship_to_address => format_to_address,
        :access_key_id => @s3_access_key_id,
        :secret_access_key => @s3_secret_access_key,
        :template_url => "http://bsgnpo.s3.amazonaws.com/template.fba.packing.slip.pdf",
        :bucket => 'bsgnpo'
      })
    end
    
    def set_shipment_status(status)
      request = Request.new({
        'Action'                                                    => 'CreateInboundShipment',
        'Marketplace'                                               => @marketplace_id,
        'SellerId'                                                  => @merchant_id,
        'AWSAccessKeyId'                                            => @access_key_id,
        'InboundShipmentHeader.ShipmentName'                        => @shipment_name,
        'InboundShipmentHeader.ShipmentStatus'                      => status,
        'InboundShipmentHeader.ShipFromAddress.Name'                => @from_address[:name],
        'InboundShipmentHeader.ShipFromAddress.AddressLine1'        => @from_address[:address_line_1],
        'InboundShipmentHeader.ShipFromAddress.AddressLine2'        => @from_address[:address_line_2],
        'InboundShipmentHeader.ShipFromAddress.City'                => @from_address[:city],
        'InboundShipmentHeader.ShipFromAddress.StateOrProvinceCode' => @from_address[:state_or_province_code],
        'InboundShipmentHeader.ShipFromAddress.PostalCode'          => @from_address[:postal_code],
        'InboundShipmentHeader.ShipFromAddress.CountryCode'         => @from_address[:country_code],
        'InboundShipmentHeader.DestinationFulfillmentCenterId'      => @fulfillment_center_id
      },@secret_access_key)
      
      response = request.post.to_hash
      shipment_id = response["CreateInboundShipmentResponse"]["CreateInboundShipmentResult"]["ShipmentId"]

      if shipment_id
        @shipment_status = status
      end
      shipment_id
    end
    
  end

end
